@@ -240,19 +240,8 @@ sub chordpro {
 	}
     }
 
-    if ( my $xc = $::config->{settings}->{transcode} ) {
-	# Set target parser for the backend so it can find the transcoded
-	# chord definitions.
-	App::Music::ChordPro::Chords::set_parser($xc);
-	# Generate the songbook.
-	$res = $pkg->generate_songbook($s);
-	# Restore parser.
-	App::Music::ChordPro::Chords::set_parser($::config->{notes}->{system} );
-    }
-    else {
-	# Generate the songbook.
-	$res = $pkg->generate_songbook($s);
-    }
+    # Call backend to produce output.
+    $res = $pkg->generate_songbook($s);
 
   WRITE_OUTPUT:
     # Some backends write output themselves, others return an
@@ -750,7 +739,7 @@ sub app_setup {
 
           ### Options ###
 
-	  "a2crd",			# perform ascii to cho
+	  "a2crd!",			# perform ascii to cho
 	  "crd",			# input is ascii, not cho
           "output|o=s",                 # Saves the output to FILE
           "generate=s",
@@ -1016,6 +1005,7 @@ Usage: $0 [ options ] [ file ... ]
 
 Options:
     --a2crd                       Perform text to ChordPro conversion only
+    --noa2crd                     Do not auto-sense text to ChordPro conversion
     --about  -A                   About ChordPro...
     --config=JSON  --cfg          Config definitions (multiple)
     --cover=FILE                  Add cover pages from PDF document
