@@ -1,19 +1,15 @@
 #! perl
 
-package main;
-
-our $config;
-
 package App::Music::ChordPro::Output::PDF::Writer;
 
 use strict;
 use warnings;
 use Encode;
+use PDF::API2;
 use Text::Layout;
 use IO::String;
 use Carp;
 
-use App::Music::ChordPro::Utils qw( expand_tilde );
 use App::Music::ChordPro::Output::Common qw( fmt_subst prep_outlines demarkup );
 
 # For regression testing, run perl with PERL_HASH_SEED set to zero.
@@ -33,10 +29,7 @@ sub new {
     $self->{pdf}->mediabox( $ps->{papersize}->[0],
 			    $ps->{papersize}->[1] );
     $self->{layout} = Text::Layout->new( $self->{pdf} );
-    $self->{tmplayout} = undef;
-
-    %fontcache = ();
-
+    %fontcache = () if $::__EMBEDDED__;
     $self;
 }
 
@@ -45,19 +38,13 @@ sub info {
     unless ( $info{CreationDate} ) {
 	my @tm = gmtime( $regtest ? $faketime : time );
 	$info{CreationDate} =
-	  sprintf("D:%04d%02d%02d%02d%02d%02d+01'00",
+	  sprintf("D:%04d%02d%02d%02d%02d%02d+00'00'",
 		  1900+$tm[5], 1+$tm[4], @tm[3,2,1,0]);
     }
-    if ( $self->{pdf}->can("info_metadata") ) {
-	for ( keys(%info) ) {
-	    $self->{pdf}->info_metadata( $_, $info{$_} );
-	}
-    }
-    else {
-	$self->{pdf}->info(%info);
-    }
+    $self->{pdf}->info( %info );
 }
 
+
 sub wrap {
     my ( $self, $text, $m ) = @_;
 
@@ -82,8 +69,8 @@ sub wrap {
 
 sub _fgcolor {
     my ( $self, $col ) = @_;
-    if ( !defined($col) || $col =~ /^foreground(?:-medium|-light)?$/ ) {
-	$col = $self->{ps}->{theme}->{$col//"foreground"};
+    if ( !defined($col) || $col eq "foreground" ) {
+	$col = $self->{ps}->{theme}->{foreground};
     }
     elsif ( $col eq "background" ) {
 	$col = $self->{ps}->{theme}->{background};
@@ -99,8 +86,8 @@ sub _bgcolor {
     if ( !defined($col) || $col eq "background" ) {
 	$col = $self->{ps}->{theme}->{background};
     }
-    elsif ( $col =~ /^foreground(?:-medium|-light)?$/ ) {
-	$col = $self->{ps}->{theme}->{$col};
+    elsif ( $col eq "foreground" ) {
+	$col = $self->{ps}->{theme}->{foreground};
     }
     elsif ( !$col ) {
 	Carp::confess("Undefined bgcolor: $col");
@@ -108,13 +95,6 @@ sub _bgcolor {
     $col;
 }
 
-sub _yflip {
-    #warn("Text::Layout = $Text::Layout::VERSION\n" );
-    $Text::Layout::VERSION gt "0.027";
-}
-
-my $yflip;
-
 sub text {
     my ( $self, $text, $x, $y, $font, $size, $nomarkup ) = @_;
 #    print STDERR ("T: @_\n");
@@ -127,43 +107,70 @@ sub text {
     $self->{layout}->{_currentcolor} = $self->_fgcolor($font->{color});
     # Watch out for regression... May have to do this in the nomarkup case only.
     if ( $nomarkup ) {
-	$text =~ s/'/\x{2019}/g;		# friendly quote
 	$self->{layout}->set_text($text);
     }
     else {
 	$self->{layout}->set_markup($text);
-	for ( @{ $self->{layout}->{_content} } ) {
-	    $_->{text} =~ s/\'/\x{2019}/g;	# friendly quote
-	}
     }
     $y -= $self->{layout}->get_baseline;
     $self->{layout}->show( $x, $y, $self->{pdftext} );
 
-    my $e = $self->{layout}->get_pixel_extents;
-    if ( ref($e) eq 'ARRAY' ) { # Text::Layout <= 0.026
-	$e = $e->[1];
-    }
-    elsif ( $yflip //= _yflip() ) {
-	$e->{y} += $e->{height};
-    }
+    my $e = ($self->{layout}->get_pixel_extents)[1];
 
     # Handle decorations (background, box).
-    my $bgcol = $self->_bgcolor($font->{background});
+    my $bgcol = $font->{background};
     undef $bgcol if $bgcol && $bgcol =~ /^no(?:ne)?$/i;
     my $debug = $ENV{CHORDPRO_DEBUG_TEXT} ? "magenta" : undef;
     my $frame = $font->{frame} || $debug;
     undef $frame if $frame && $frame =~ /^no(?:ne)?$/i;
     if ( $bgcol || $frame ) {
-	printf("BB: %.2f %.2f %.2f %.2f\n", @{$e}{qw( x y width height ) } )
-	  if $debug;
+	#printf("BB: %.2f %.2f %.2f %.2f\n", @{$e}{qw( x y width height ) } );
 	# Draw background and.or frame.
 	my $d = $debug ? 0 : 1;
 	$frame = $debug || $font->{color} || $self->{ps}->{theme}->{foreground} if $frame;
-	# $self->crosshair( $x, $y, 20, 0.2, "magenta" );
 	$self->rectxy( $x + $e->{x} - $d,
 		       $y + $e->{y} + $d,
 		       $x + $e->{x} + $e->{width} + $d,
-		       $y + $e->{y} - $e->{height} - $d,
+		       $y - $e->{height} - $d,
+		       0.5, $bgcol, $frame);
+    }
+
+    $x += $e->{width};
+#    print STDERR ("TX: $x\n");
+    return $x;
+}
+
+# Identical copy of text, but without baseline correction.
+sub text_nobl {
+    my ( $self, $text, $x, $y, $font, $size ) = @_;
+#    print STDERR ("T: @_\n");
+    $font ||= $self->{font};
+    $size ||= $font->{size};
+
+    $self->{layout}->set_font_description($font->{fd});
+    $self->{layout}->set_font_size($size);
+    # We don't have set_color in the API.
+    $self->{layout}->{_currentcolor} = $self->_fgcolor($font->{color});
+    $self->{layout}->set_markup($text);
+    $self->{layout}->show( $x, $y, $self->{pdftext} );
+
+    my $e = ($self->{layout}->get_pixel_extents)[1];
+
+    # Handle decorations (background, box).
+    my $bgcol = $font->{background};
+    undef $bgcol if $bgcol && $bgcol =~ /^no(?:ne)?$/i;
+    my $debug = "blue";
+    my $frame = $font->{frame} || $debug;
+    undef $frame if $frame && $frame =~ /^no(?:ne)?$/i;
+    if ( $bgcol || $frame ) {
+	#printf("BB: %.2f %.2f %.2f %.2f\n", @{$e}{qw( x y width height ) } );
+	# Draw background and.or frame.
+	my $d = $debug ? 0 : 1;
+	$frame = $debug || $font->{color} || $self->{ps}->{theme}->{foreground} if $frame;
+	$self->rectxy( $x + $e->{x} - $d,
+		       $y + $e->{y} + $d,
+		       $x + $e->{x} + $e->{width} + $d,
+		       $y - $e->{height} - $d,
 		       0.5, $bgcol, $frame);
     }
 
@@ -181,26 +188,28 @@ sub setfont {
     $self->{pdftext}->font( $font->{fd}->{font}, $size );
 }
 
+my $tmplayout;
+
 sub strwidth {
     my ( $self, $text, $font, $size ) = @_;
     $font ||= $self->{font};
     $size ||= $self->{fontsize} || $font->{size};
-    $self->{tmplayout} //= Text::Layout->new( $self->{pdf} );
-    $self->{tmplayout}->set_font_description($font->{fd});
-    $self->{tmplayout}->set_font_size($size);
-    $self->{tmplayout}->set_markup($text);
-    $self->{tmplayout}->get_pixel_size->{width};
+    $tmplayout //= Text::Layout->new( $self->{pdf} );
+    $tmplayout->set_font_description($font->{fd});
+    $tmplayout->set_font_size($size);
+    $tmplayout->set_markup($text);
+    $tmplayout->get_pixel_size->{width};
 }
 
 sub strheight {
     my ( $self, $text, $font, $size ) = @_;
     $font ||= $self->{font};
     $size ||= $self->{fontsize} || $font->{size};
-    $self->{tmplayout} //= Text::Layout->new( $self->{pdf} );
-    $self->{tmplayout}->set_font_description($font->{fd});
-    $self->{tmplayout}->set_font_size($size);
-    $self->{tmplayout}->set_markup($text);
-    $self->{tmplayout}->get_pixel_size->{height};
+    $tmplayout //= Text::Layout->new( $self->{pdf} );
+    $tmplayout->set_font_description($font->{fd});
+    $tmplayout->set_font_size($size);
+    $tmplayout->set_markup($text);
+    $tmplayout->get_pixel_size->{height};
 }
 
 sub line {
@@ -217,12 +226,11 @@ sub line {
 }
 
 sub hline {
-    my ( $self, $x, $y, $w, $lw, $color, $cap ) = @_;
-    $cap //= 2;
+    my ( $self, $x, $y, $w, $lw, $color ) = @_;
     my $gfx = $self->{pdfgfx};
     $gfx->save;
     $gfx->strokecolor( $self->_fgcolor($color) );
-    $gfx->linecap($cap);
+    $gfx->linecap(2);
     $gfx->linewidth($lw||1);
     $gfx->move( $x, $y );
     $gfx->hline( $x + $w );
@@ -231,12 +239,11 @@ sub hline {
 }
 
 sub vline {
-    my ( $self, $x, $y, $h, $lw, $color, $cap ) = @_;
-    $cap //= 2;
+    my ( $self, $x, $y, $h, $lw, $color ) = @_;
     my $gfx = $self->{pdfgfx};
     $gfx->save;
     $gfx->strokecolor( $self->_fgcolor($color) );
-    $gfx->linecap($cap);
+    $gfx->linecap(2);
     $gfx->linewidth($lw||1);
     $gfx->move( $x, $y );
     $gfx->vline( $y - $h );
@@ -305,42 +312,22 @@ sub cross {
     $gfx->restore;
 }
 
-sub crosshair {			# for debugging
-    my ( $self, $x, $y, $r, $lw, $strokecolor ) = @_;
-    my $gfx = $self->{pdfgfx};
-    $gfx->save;
-    $gfx->strokecolor($self->_fgcolor($strokecolor)) if $strokecolor;
-    $gfx->linewidth($lw||1);
-    $gfx->move( $x, $y - $r );
-    $gfx->line( $x, $y + $r );
-    $gfx->stroke if $strokecolor;
-    $gfx->move( $x - $r, $y );
-    $gfx->line( $x + $r, $y );
-    $gfx->stroke if $strokecolor;
-    $gfx->restore;
-}
-
 sub get_image {
-    my ( $self, $elt ) = @_;
+    my ( $self, $uri ) = @_;
 
     my $img;
-    my $uri = $elt->{uri};
-    warn("get_image($uri)\n") if $config->{debug}->{images};
     if ( $uri =~ /^id=(.+)/ ) {
 	my $a = $App::Music::ChordPro::Output::PDF::assets->{$1};
-
-	if ( $a->{type} eq "abc" ) {
-	    my $res = App::Music::ChordPro::Output::PDF::abc2image( undef, $self, $a );
-	    return $self->get_image( { %$elt, uri => $res->{src} } );
-	}
-	elsif ( $a->{type} eq "jpg" ) {
-	    $img = $self->{pdf}->image_jpeg(IO::String->new($a->{data}));
+	my $d = $a->{data};
+	my $fh = IO::String->new($d);
+	if ( $a->{type} eq "jpg" ) {
+	    $img = $self->{pdf}->image_jpeg($fh);
 	}
 	elsif ( $a->{type} eq "png" ) {
-	    $img = $self->{pdf}->image_png(IO::String->new($a->{data}));
+	    $img = $self->{pdf}->image_png($fh);
 	}
 	elsif ( $a->{type} eq "gif" ) {
-	    $img = $self->{pdf}->image_gif(IO::String->new($a->{data}));
+	    $img = $self->{pdf}->image_gif($fh);
 	}
 	return $img;
     }
@@ -373,7 +360,6 @@ sub newpage {
     $self->{pdfpage} = $self->{pdf}->page($page||0);
     $self->{pdfpage}->mediabox( $ps->{papersize}->[0],
 				$ps->{papersize}->[1] );
-
     $self->{pdfgfx}  = $self->{pdfpage}->gfx;
     $self->{pdftext} = $self->{pdfpage}->text;
     unless ($ps->{theme}->{background} =~ /^white|none|#ffffff$/i ) {
@@ -389,55 +375,12 @@ sub newpage {
     }
 }
 
-sub openpage {
-    my ( $self, $ps, $page ) = @_;
-    $self->{pdfpage} = $self->{pdf}->openpage($page);
-    $self->{pdfgfx}  = $self->{pdfpage}->gfx;
-    $self->{pdftext} = $self->{pdfpage}->text;
-}
-
-sub importpage {
-    my ( $self, $fn, $pg ) = @_;
-    my $bg = $self->{pdfapi}->open($fn);
-    return unless $bg;		# should have been checked
-    $pg = $bg->pages if $pg > $bg->pages;
-    $self->{pdf}->import_page( $bg, $pg, $self->{pdfpage} );
-    # Make sure the contents get on top of it.
-    $self->{pdfgfx}  = $self->{pdfpage}->gfx;
-    $self->{pdftext} = $self->{pdfpage}->text;
-}
-
-sub importfile {
-    my ( $self, $filename ) = @_;
-    my $pdf = $self->{pdfapi}->open($filename);
-    return unless $pdf;		# should have been checked
-    for ( my $page = 1; $page <= $pdf->pages; $page++ ) {
-	$self->{pdf}->import_page( $pdf, $page );
-    }
-    return { pages => $pdf->pages, $pdf->info_metadata };
-}
-
 sub pagelabel {
     my ( $self, $page, $style, $prefix ) = @_;
-    $style //= 'arabic';
-
-    # PDF::API2 2.042 has some incompatible changes...
-    my $c = $self->{pdf}->can("page_labels");
-    if ( $c ) {			# 2.042+
-	my $opts = { style => $style eq 'Roman' ? 'R' :
-		              $style eq 'roman' ? 'r' :
-                              $style eq 'Alpha' ? 'A' :
-                              $style eq 'alpha' ? 'a' : 'D',
-		     defined $prefix ? ( prefix => $prefix ) : (),
-		     start => 1 };
-	$c->( $self->{pdf}, $page, $opts );
-    }
-    else {
-	my $opts = { -style => $style,
-		     defined $prefix ? ( -prefix => $prefix ) : (),
-		     -start => 1 };
-	$self->{pdf}->pageLabel( $page, $opts );
-    }
+    my $opts = { -style => $style // 'arabic',
+		 defined $prefix ? ( -prefix => $prefix ) : (),
+		 -start => 1 };
+    $self->{pdf}->pageLabel( $page, $opts );
 }
 
 sub make_outlines {
@@ -503,12 +446,7 @@ sub make_outlines {
 		}
 		# Display info.
 		$ol->title( demarkup( fmt_subst( $song, $ctl->{line} ) ) );
-		if ( my $c = $ol->can("destination") ) {
-		    $c->( $ol, $pdf->openpage( $song->{meta}->{tocpage} + $start ) );
-		}
-		else {
-		    $ol->dest($pdf->openpage( $song->{meta}->{tocpage} + $start ));
-		}
+		$ol->dest($pdf->openpage( $song->{meta}->{tocpage} + $start ));
 	    }
 	}
     }
@@ -532,7 +470,7 @@ sub init_fonts {
     my $ps = $self->{ps};
     my $fail;
 
-    my $fc = Text::Layout::FontConfig->new( debug => $config->{debug}->{fonts} > 1 );
+    my $fc = Text::Layout::FontConfig->new;
 
     # Add font dirs.
     my @d = ( @{$ps->{fontdir}}, ::rsc_or_file("fonts/"), $ENV{FONTDIR} );
@@ -540,7 +478,6 @@ sub init_fonts {
     splice( @d, -2, 1 ) if $d[-2] eq "fonts/";
     for my $fontdir ( @d ) {
 	next unless $fontdir;
-	$fontdir = expand_tilde($fontdir);
 	if ( -d $fontdir ) {
 	    $self->{pdfapi}->can("addFontDirs")->($fontdir);
 	    $fc->add_fontdirs($fontdir);
@@ -551,10 +488,6 @@ sub init_fonts {
 	}
     }
 
-    # Make sure we have this one.
-    $fc->register_font( "ChordProSymbols.ttf", "chordprosymbols", "", {} );
-
-    # Process the fontconfig.
     foreach my $ff ( keys( %{ $ps->{fontconfig} } ) ) {
 	my @fam = split( /\s*,\s*/, $ff );
 	foreach my $s ( keys( %{ $ps->{fontconfig}->{$ff} } ) ) {
@@ -600,7 +533,7 @@ sub init_pangofont {
     my $ps = $self->{ps};
     my $font = $ps->{fonts}->{$ff};
 
-    my $fc = Text::Layout::FontConfig->new( debug => $config->{debug}->{fonts} > 1 );
+    my $fc = Text::Layout::FontConfig->new;
     eval {
 	$font->{fd} = $fc->from_string($font->{description});
 	$font->{fd}->get_font($self->{layout}); # force load
@@ -618,9 +551,9 @@ sub init_filefont {
     my $ps = $self->{ps};
     my $font = $ps->{fonts}->{$ff};
 
-    my $fc = Text::Layout::FontConfig->new( debug => $config->{debug}->{fonts} > 1 );
+    my $fc = Text::Layout::FontConfig->new;
     eval {
-	my $t = $fc->from_filename(expand_tilde($font->{file}));
+	my $t = $fc->from_filename($font->{file});
 	$t->get_font($self->{layout}); # force load
 	$t->{font}->{Name}->{val} =~ s/~.*/~$faketime/ if $regtest;
 	$t->{_ff} = $ff;
@@ -634,12 +567,10 @@ sub init_corefont {
 
     my $ps = $self->{ps};
     my $font = $ps->{fonts}->{$ff};
-    my $cf = App::Music::ChordPro::Output::PDF::is_corefont($font->{name});
-    die("Config error: \"$font->{name}\" is not a built-in font\n")
-      unless $cf;
-    my $fc = Text::Layout::FontConfig->new( debug => $config->{debug}->{fonts} > 1 );
+
+    my $fc = Text::Layout::FontConfig->new;
     eval {
-	$font->{fd} = $fc->from_filename($cf);
+	$font->{fd} = $fc->from_filename($font->{name});
 	$font->{fd}->get_font($self->{layout}); # force load
 	$font->{_ff} = $ff;
     };
@@ -654,7 +585,6 @@ sub show_vpos {
 use File::Temp;
 
 my $cname;
-my $rname;
 sub embed {
     my ( $self, $file ) = @_;
     return unless -f $file;
@@ -680,21 +610,13 @@ sub embed {
     $a->movie($cname, "ChordProConfig" );
     $a->open(0);
 
-    my $rf;
-    if ( $rname ) {
-	open( $rf, '>', $rname );
-    }
-    else {
-	( $rf, $rname ) = File::Temp::tempfile( UNLINK => 0);
-    }
-    binmode( $rf, ':utf8' );
-    open( $rf, '>', $rname );
-    binmode( $rf, ':utf8' );
-    print $rf (::runtimeinfo());
-    close($rf);
+    open( $cf, '>', $cname );
+    binmode( $cf, ':utf8' );
+    print $cf (::runtimeinfo());
+    close($cf);
 
     $a = $self->{pdfpage}->annotation();
-    $a->movie($rname, "ChordProRunTime" );
+    $a->movie($cname, "ChordProRunTime" );
     $a->open(0);
 }
 
@@ -702,8 +624,6 @@ END {
     return unless $cname;
     unlink($cname);
     undef $cname;
-    unlink($rname);
-    undef $rname;
 }
 
 1;
